# Copyright (C) 2021-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: AGPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from dataclasses import FrozenInstanceError
from unittest import TestCase

from notus.scanner.errors import PackageError
from notus.scanner.models.packages.deb import DEBPackage
from notus.scanner.models.packages.package import (
    Package,
    PackageAdvisories,
    PackageAdvisory,
    PackageComparison,
    PackageType,
)
from notus.scanner.models.packages.rpm import RPMPackage


class PackageTestCase(TestCase):
    def test_matching_hashes(self):
        """hashes for the same package should match"""
        package1 = Package(
            name="foo-bar",
            full_name="foo-bar-1.2.3-4.x86_64",
            full_version="1.2.3-4.x86_64",
        )
        package2 = Package(
            name="foo-bar",
            full_name="foo-bar-1.2.3-4.x86_64",
            full_version="1.2.3-4.x86_64",
        )

        self.assertEqual(hash(package1), hash(package2))

    def test_not_matching_hashes(self):
        """hashes for different packages should not match"""
        package1 = Package(
            name="foo-bar",
            full_name="foo-bar-1.2.3-4.x86_64",
            full_version="1.2.3-4.x86_64",
        )
        package2 = Package(
            name="foo-bar",
            full_name="foo-bar-1.2.4-4.x86_64",
            full_version="1.2.4-4.x86_64",
        )

        self.assertNotEqual(hash(package1), hash(package2))

    def test_gt_package_error(self):
        package1 = RPMPackage(
            name="foo-bar",
            version="1.2.3",
            release="4",
            arch="x86_64",
            full_name="foo-bar-1.2.3-4.x86_64",
            full_version="1.2.3-4.x86_64",
        )
        package2 = DEBPackage(
            name="foo-bar",
            epoch="1",
            upstream_version="1.2.4",
            debian_revision="4",
            full_name="foo-bar-1.2.4-4.x86_64",
            full_version="1.2.4-4.x86_64",
        )
        with self.assertRaises(PackageError):
            if package1 > package2:
                self.fail("PackageError should occur")

    def test_version_compare(self):
        version_a = "1.2.3"
        version_b = "1.2.3"
        ret = Package.version_compare(version_a, version_b)
        self.assertEqual(ret, PackageComparison.EQUAL)

        version_a = "1.2.3"
        version_b = "1.2.12"
        ret = Package.version_compare(version_a, version_b)
        self.assertEqual(ret, PackageComparison.B_NEWER)

        version_a = "1.2.3"
        version_b = "1.2.3a"
        ret = Package.version_compare(version_a, version_b)
        self.assertEqual(ret, PackageComparison.B_NEWER)

        version_a = "1.2.3~rc0"
        version_b = "1.2.3"
        ret = Package.version_compare(version_a, version_b)
        self.assertEqual(ret, PackageComparison.B_NEWER)

        version_a = "1.2.3a"
        version_b = "1.2.3b"
        ret = Package.version_compare(version_a, version_b)
        self.assertEqual(ret, PackageComparison.B_NEWER)

        version_a = "1.2.3a"
        version_b = "1.2.3-2"
        ret = Package.version_compare(version_a, version_b)
        self.assertEqual(ret, PackageComparison.B_NEWER)

        version_a = "1.2"
        version_b = "1.2.3"
        ret = Package.version_compare(version_a, version_b)
        self.assertEqual(ret, PackageComparison.B_NEWER)

        version_a = "1.1.1c"
        version_b = "1.1.1k"
        ret = Package.version_compare(version_a, version_b)
        self.assertEqual(ret, PackageComparison.B_NEWER)

        version_a = "1.2.3.1"
        version_b = "1.2.3_a"
        ret = Package.version_compare(version_a, version_b)
        self.assertEqual(ret, PackageComparison.B_NEWER)

        version_a = "1.2.3_a"
        version_b = "1.2.3_1"
        ret = Package.version_compare(version_a, version_b)
        self.assertEqual(ret, PackageComparison.B_NEWER)

        version_a = "20211016ubuntu0.20.04.1"
        version_b = "20211016~20.04.1"
        ret = Package.version_compare(version_a, version_b)
        self.assertEqual(ret, PackageComparison.A_NEWER)
        ret = Package.version_compare(version_a=version_b, version_b=version_a)
        self.assertEqual(ret, PackageComparison.B_NEWER)


class PackageAdvisoryTestCase(TestCase):
    def test_constructor(self):
        package = RPMPackage.from_full_name(
            "foo-1.2.3-3.aarch64",
        )
        advisory = "1.2.3.4.5"

        package_advisory = PackageAdvisory(
            package=package,
            oid=advisory,
            symbol=">=",
            is_vulnerable=lambda _: False,
        )

        self.assertEqual(package_advisory.package, package)
        self.assertEqual(package_advisory.oid, advisory)

    def test_immutability(self):
        package1 = RPMPackage.from_full_name(
            "foo-1.2.3-3.aarch64",
        )
        package2 = RPMPackage.from_full_name(
            "bar-1.2.3-3.aarch64",
        )
        advisory1 = "1.2.3.4.5"
        advisory2 = "1.2.3.4.6"

        package_advisory = PackageAdvisory(
            package=package1,
            oid=advisory1,
            symbol=">=",
            is_vulnerable=lambda _: False,
        )

        with self.assertRaises(FrozenInstanceError):
            package_advisory.package = package2

        with self.assertRaises(FrozenInstanceError):
            package_advisory.oid = advisory2

        self.assertEqual(package_advisory.package, package1)
        self.assertEqual(package_advisory.oid, advisory1)

    def test_equal(self):
        package1 = RPMPackage.from_full_name(
            "foo-1.2.3-3.aarch64",
        )
        package2 = RPMPackage.from_full_name(
            "bar-1.2.3-3.aarch64",
        )
        advisory1 = "1.2.3.4.5"
        advisory2 = "1.2.3.4.6"

        package_advisory1 = PackageAdvisory(
            package=package1,
            oid=advisory1,
            symbol=">=",
            is_vulnerable=lambda _: False,
        )
        package_advisory2 = PackageAdvisory(
            package=package2,
            oid=advisory2,
            symbol=">=",
            is_vulnerable=lambda _: False,
        )
        package_advisory3 = PackageAdvisory(
            package=package1,
            oid=advisory1,
            symbol=">=",
            is_vulnerable=lambda _: False,
        )
        package_advisory4 = PackageAdvisory(
            package=package1,
            oid=advisory2,
            symbol=">=",
            is_vulnerable=lambda _: False,
        )

        self.assertEqual(package_advisory1, package_advisory1)
        self.assertEqual(package_advisory1, package_advisory3)
        self.assertNotEqual(package_advisory2, package_advisory3)
        self.assertNotEqual(package_advisory1, package_advisory2)
        self.assertNotEqual(package_advisory1, package_advisory4)
        self.assertNotEqual(package_advisory2, package_advisory4)

    def test_hash(self):
        package1 = RPMPackage.from_full_name(
            "foo-1.2.3-3.aarch64",
        )
        package2 = RPMPackage.from_full_name(
            "bar-1.2.3-3.aarch64",
        )
        advisory1 = "1.2.3.4.5"

        advisory2 = "1.2.3.4.6"

        package_advisory1 = PackageAdvisory(
            package=package1,
            oid=advisory1,
            symbol=">=",
            is_vulnerable=lambda _: False,
        )
        package_advisory2 = PackageAdvisory(
            package=package2,
            oid=advisory2,
            symbol=">=",
            is_vulnerable=lambda _: False,
        )
        package_advisory3 = PackageAdvisory(
            package=package1,
            oid=advisory1,
            symbol=">=",
            is_vulnerable=lambda _: False,
        )
        package_advisory4 = PackageAdvisory(
            package=package1,
            oid=advisory2,
            symbol=">=",
            is_vulnerable=lambda _: False,
        )

        self.assertEqual(hash(package_advisory1), hash(package_advisory1))
        self.assertEqual(hash(package_advisory1), hash(package_advisory3))
        self.assertNotEqual(hash(package_advisory1), hash(package_advisory2))
        self.assertNotEqual(hash(package_advisory3), hash(package_advisory2))
        self.assertNotEqual(hash(package_advisory1), hash(package_advisory4))
        self.assertNotEqual(hash(package_advisory2), hash(package_advisory4))


class PackageAdvisoriesTestCase(TestCase):
    def test_constructor(self):
        package_advisories = PackageAdvisories(PackageType.DEB)

        self.assertIsInstance(package_advisories.advisories, dict)
        self.assertEqual(len(package_advisories), 0)

    def test_add_advisory_for_package(self):
        package_advisories = PackageAdvisories(PackageType.RPM)
        package = RPMPackage.from_full_name(
            "foo-1.2.3-3.aarch64",
        )
        advisory = "1.2.3.4.5"

        self.assertEqual(len(package_advisories), 0)

        package_advisories.add_advisory_for_package(package, advisory, None)

        self.assertEqual(len(package_advisories), 1)

    def test_default_is_vulnerable(self):
        package_advisories = PackageAdvisories(PackageType.RPM)
        package = RPMPackage.from_full_name(
            "foo-1.2.3-3.aarch64",
        )
        advisory = "1.2.3.4.5"

        other = RPMPackage.from_full_name(
            "foo-1.2.4-3.aarch64",
        )
        package_advisories.add_advisory_for_package(package, advisory, None)
        advisories = package_advisories.get_package_advisories_for_package(
            other
        )
        self.assertEqual(1, len(advisories))
        for package_advisories in advisories.values():
            for adv in package_advisories:
                self.assertFalse(adv.is_vulnerable(other))

    def test_add_duplicate_advisory_for_package(self):
        package_advisories = PackageAdvisories(package_type=PackageType.RPM)
        package1 = RPMPackage.from_full_name(
            "foo-1.2.3-3.aarch64",
        )
        package2 = RPMPackage.from_full_name(
            "foo-1.2.3-3.aarch64",
        )
        advisory1 = "1.2.3.4.5"
        advisory2 = "1.2.3.4.5"

        self.assertEqual(len(package_advisories), 0)

        package_advisories.add_advisory_for_package(package1, advisory1, None)

        self.assertEqual(len(package_advisories), 1)

        package_advisories.add_advisory_for_package(package1, advisory1, None)

        self.assertEqual(len(package_advisories), 1)

        package_advisories.add_advisory_for_package(package1, advisory2, None)

        self.assertEqual(len(package_advisories), 1)

        package_advisories.add_advisory_for_package(package2, advisory1, None)

        self.assertEqual(len(package_advisories), 1)

        package_advisories.add_advisory_for_package(package2, advisory2, None)

    def test_get_package_advisories_for_package(self):
        package_advisories = PackageAdvisories(package_type=PackageType.RPM)
        package1 = RPMPackage.from_full_name(
            "foo-1.2.3-3.aarch64",
        )
        package2 = RPMPackage.from_full_name(
            "bar-1.2.3-3.aarch64",
        )
        advisory1 = "1.2.3.4.5"
        advisory2 = "1.2.3.4.6"

        self.assertEqual(len(package_advisories), 0)

        package_advisories.add_advisory_for_package(package1, advisory1, None)
        package_advisories.add_advisory_for_package(package1, advisory2, None)
        package_advisories.add_advisory_for_package(package2, advisory2, None)

        advisories1 = package_advisories.get_package_advisories_for_package(
            package1
        )
        advisories2 = package_advisories.get_package_advisories_for_package(
            package2
        )
        self.assertEqual(len(advisories1), 2)
        self.assertEqual(len(advisories2), 1)



# Copyright (C) 2021-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: AGPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from unittest import TestCase

from notus.scanner.models.packages.ebuild import EBuildPackage


class EBuildPackageTestCase(TestCase):
    def test_parse_fullname(self):
        """
        verifies if ebuild id able to parse the fullname examples provided by
        gentoo_examples.txt
        """
        currentp = Path(__file__).parent
        with (currentp / "gentoo_examples.txt").open(
            "r", encoding="utf-8"
        ) as file:
            fullname = file.readline()
            if not EBuildPackage.from_full_name(fullname):
                self.fail(f"{fullname} is not parsable for EBuildPackage.")

    def test_guard(self):
        self.assertIsNone(EBuildPackage.from_full_name(""))
        self.assertIsNone(EBuildPackage.from_full_name("www-servers/"))
        self.assertIsNone(EBuildPackage.from_full_name("www-servers/name"))
        self.assertIsNone(EBuildPackage.from_name_and_full_version("", "1.2.3"))
        self.assertIsNone(EBuildPackage.from_name_and_full_version("name", ""))

    def test_comparability(self):
        apache1 = EBuildPackage.from_full_name("www-servers/apache-2.4.51-r2")
        apache2 = EBuildPackage.from_name_and_full_version(
            "www-servers/apache", "2.4.51-r3"
        )
        if not apache1 or not apache2:
            self.fail("Unable to parse packages")
        self.assertGreater(apache2, apache1)
        self.assertLess(apache1, apache2)
        apache3 = EBuildPackage.from_name_and_full_version(
            "www-servers/apache", "2.4.51-r3"
        )
        if not apache3:
            self.fail("Unable to parse package")
        self.assertGreaterEqual(apache2, apache3)
        self.assertLessEqual(apache2, apache3)
        apache4 = EBuildPackage.from_name_and_full_version(
            "apache", "2.4.51-r3"
        )
        if not apache4:
            self.fail("Unable to parse package")
        # www-servers is a part of the name hence not comparable
        self.assertNotEqual(apache4, apache3)



# Copyright (C) 2021-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: AGPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from pathlib import Path
from unittest import TestCase

from notus.scanner.errors import AdvisoriesLoadingError
from notus.scanner.loader.gpg_sha_verifier import VerificationResult
from notus.scanner.loader.json import JSONAdvisoriesLoader
from notus.scanner.models.packages.rpm import RPMPackage

_here = Path(__file__).parent


class JSONAdvisoriesLoaderTestCase(TestCase):
    def test_unknown_file(self):
        loader = JSONAdvisoriesLoader(
            advisories_directory_path=_here,
            verify=lambda _: VerificationResult.SUCCESS,
        )

        self.assertIsNone(loader.load_package_advisories("foo"))

    def test_verification_failure(self):
        loader = JSONAdvisoriesLoader(
            advisories_directory_path=_here,
            verify=lambda _: VerificationResult.INVALID_HASH,
        )
        with self.assertRaises(AdvisoriesLoadingError):
            loader.load_package_advisories("EmptyOS")

    def test_empty_file(self):
        loader = JSONAdvisoriesLoader(
            advisories_directory_path=_here,
            verify=lambda _: VerificationResult.SUCCESS,
        )

        advisories = loader.load_package_advisories("EmptyOS")
        self.assertIsNone(advisories)

    def test_example(self):
        loader = JSONAdvisoriesLoader(
            advisories_directory_path=_here,
            verify=lambda _: VerificationResult.SUCCESS,
        )

        advisories = loader.load_package_advisories("EulerOS V2.0SP1")
        if not advisories:
            self.fail("Advisories are none")
        self.assertIsNotNone(advisories)
        self.assertEqual(len(advisories), 55)

        package1 = RPMPackage.from_full_name("openssh-6.6.1p1-25.4.h3.x86_64")
        if not package1:
            self.fail("package1 is None")
        package2 = RPMPackage.from_full_name(
            "openssh-clients-6.6.1p1-25.4.h3.x86_64"
        )

        if not package2:
            self.fail("package2 is None")
        package_advisories1 = advisories.get_package_advisories_for_package(
            package1
        )
        package_advisories2 = advisories.get_package_advisories_for_package(
            package2
        )

        oid = "1.3.6.1.4.1.25623.1.1.2.2016.1008"

        self.assertEqual(len(package_advisories1), 1)
        self.assertEqual(len(package_advisories2), 1)

        self.assertIn(oid, package_advisories1.keys())
        self.assertIn(oid, package_advisories2.keys())

        package_advisories1 = package_advisories1[oid]
        package_advisories2 = package_advisories2[oid]

        # get first PackageAdvisory from the sets
        package_advisory1 = next(iter(package_advisories1))
        package_advisory2 = next(iter(package_advisories2))

        self.assertEqual(package_advisory1.oid, package_advisory2.oid)

        advisory = package_advisory1.oid

        self.assertEqual(advisory, "1.3.6.1.4.1.25623.1.1.2.2016.1008")

    def test_invalid_package_type(self):
        loader = JSONAdvisoriesLoader(
            advisories_directory_path=_here,
            verify=lambda _: VerificationResult.SUCCESS,
        )

        advisory = loader.load_package_advisories("invalid_package")

        self.assertIsNone(advisory)



# Copyright (C) 2021-2022 Greenbone Networks GmbH
#
# SPDX-License-Identifier: AGPL-3.0-or-later
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as
# published by the Free Software Foundation, either version 3 of the
# License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
#
# You should have received a copy of the GNU Affero General Public License
# along with this program. If not, see <http://www.gnu.org/licenses/>.

from datetime import datetime, timezone
from unittest import TestCase
from uuid import UUID

from notus.scanner.errors import MessageParsingError
from notus.scanner.messages.message import Message, MessageType


class MessageTestCase(TestCase):
    def test_default_constructor(self):
        message = Message()

        self.assertIsInstance(message.message_id, UUID)
        self.assertIsInstance(message.group_id, str)
        self.assertIsInstance(message.created, datetime)

    def test_serialize(self):
        created = datetime.fromtimestamp(1628512774)
        message_id = UUID("63026767-029d-417e-9148-77f4da49f49a")
        group_id = UUID("866350e8-1492-497e-b12b-c079287d51dd")
        message = Message(
            message_id=message_id, group_id=group_id, created=created
        )

        serialized = message.serialize()
        self.assertEqual(serialized["created"], 1628512774.0)
        self.assertEqual(
            serialized["message_id"], "63026767-029d-417e-9148-77f4da49f49a"
        )
        self.assertEqual(
            serialized["group_id"], "866350e8-1492-497e-b12b-c079287d51dd"
        )
        self.assertIsNone(message.message_type)

    def test_deserialize(self):
        data = {
            "message_id": "63026767-029d-417e-9148-77f4da49f49a",
            "group_id": "866350e8-1492-497e-b12b-c079287d51dd",
            "created": 1628512774.0,
            "message_type": "scan.start",
        }

        Message.message_type = MessageType.SCAN_START  # hack a message type

        message = Message.deserialize(data)
        self.assertEqual(
            message.message_id, UUID("63026767-029d-417e-9148-77f4da49f49a")
        )
        self.assertEqual(
            message.group_id, "866350e8-1492-497e-b12b-c079287d51dd"
        )
        self.assertEqual(
            message.created,
            datetime.fromtimestamp(1628512774.0, tz=timezone.utc),
        )

        Message.message_type = None

    def test_deserialize_no_message_type(self):
        data = {
            "message_id": "63026767-029d-417e-9148-77f4da49f49a",
            "group_id": "866350e8-1492-497e-b12b-c079287d51dd",
            "created": 1628512774.0,
        }

        with self.assertRaisesRegex(
            MessageParsingError,
            "error while parsing 'message_type', None is not a valid"
            " MessageType",
        ):
            Message.deserialize(data)

    def test_deserialize_unknown_message_type(self):
        data = {
            "message_id": "63026767-029d-417e-9148-77f4da49f49a",
            "group_id": "866350e8-1492-497e-b12b-c079287d51dd",
            "created": 1628512774.0,
            "message_type": "foo",
        }

        with self.assertRaisesRegex(
            MessageParsingError,
            "error while parsing 'message_type', 'foo' is not a valid"
            " MessageType",
        ):
            Message.deserialize(data)

    def test_to_str(self):
        created = datetime.fromtimestamp(1628512774)
        message_id = UUID("63026767-029d-417e-9148-77f4da49f49a")
        group_id = UUID("866350e8-1492-497e-b12b-c079287d51dd")
        message = Message(
            message_id=message_id, group_id=group_id, created=created
        )

        self.assertEqual(
            str(message),
            '{"message_id": "63026767-029d-417e-9148-77f4da49f49a", '
            '"message_type": null, '
            '"group_id": "866350e8-1492-497e-b12b-c079287d51dd", '
            '"created": 1628512774.0}',
        )

    def test_load(self):
        payload = (
            '{"message_id": "63026767-029d-417e-9148-77f4da49f49a", '
            '"message_type": "scan.start", '
            '"group_id": "866350e8-1492-497e-b12b-c079287d51dd", '
            '"created": 1628512774.0}'
        )

        Message.message_type = MessageType.SCAN_START  # hack a message type
        message = Message.load(payload)

        self.assertEqual(
            message.message_id, UUID("63026767-029d-417e-9148-77f4da49f49a")
        )
        self.assertEqual(
            message.group_id, "866350e8-1492-497e-b12b-c079287d51dd"
        )
        self.assertEqual(
            message.created,
            datetime.fromtimestamp(1628512774.0, tz=timezone.utc),
        )

        Message.message_type = None

    def test_load_message_id_unvalid(self):
        payload = (
            '{"message_id": "foo", '
            '"message_type": "scan.start", '
            '"group_id": "866350e8-1492-497e-b12b-c079287d51dd", '
            '"created": 1628512774.0}'
        )
        Message.message_type = MessageType.SCAN_START
        with self.assertRaises(MessageParsingError):
            Message.load(payload)
        Message.message_type = None

    def test_load_message_id_missing(self):
        payload = (
            '{"message_type": "scan.start", '
            '"group_id": "866350e8-1492-497e-b12b-c079287d51dd", '
            '"created": 1628512774.0}'
        )
        Message.message_type = MessageType.SCAN_START
        with self.assertRaises(MessageParsingError):
            Message.load(payload)
        Message.message_type = None

    def test_load_message_type_missing(self):
        payload = (
            '{"message_id": "63026767-029d-417e-9148-77f4da49f49a", '
            '"group_id": "866350e8-1492-497e-b12b-c079287d51dd", '
            '"created": 1628512774.0}'
        )
        Message.message_type = MessageType.SCAN_START
        with self.assertRaises(MessageParsingError):
            Message.load(payload)
        Message.message_type = None

    def test_load_created_unvalid(self):
        payload = (
            '{"message_id": "63026767-029d-417e-9148-77f4da49f49a", '
            '"message_type": "scan.start", '
            '"group_id": "866350e8-1492-497e-b12b-c079287d51dd", '
            '"created": "a"}'
        )
        Message.message_type = MessageType.SCAN_START
        with self.assertRaises(MessageParsingError):
            Message.load(payload)
        Message.message_type = None

    def test_load_created_missing(self):
        payload = (
            '{"message_id": "63026767-029d-417e-9148-77f4da49f49a", '
            '"message_type": "scan.start", '
            '"group_id": "866350e8-1492-497e-b12b-c079287d51dd"}'
        )
        Message.message_type = MessageType.SCAN_START
        with self.assertRaises(MessageParsingError):
            Message.load(payload)
        Message.message_type = None
